==> decoupled_toolbox-1.3.x-dev.tar.gz: decoupled_toolbox/modules/openapi_decoupled_toolbox/src/Plugin/openapi/OpenApiGenerator/DecoupledToolboxGenerator.php <==
<?php

namespace Drupal\openapi_decoupled_toolbox\Plugin\openapi\OpenApiGenerator;

use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\decoupled_toolbox\Plugin\Field\FieldFormatter\DecoupledFormatterBase;
use Drupal\openapi\Plugin\openapi\OpenApiGeneratorBase;
use Drupal\openapi_decoupled_toolbox\Entity\OpenApiDecoupledToolbox;
use Drupal\openapi_decoupled_toolbox\OpenapiDecoupledToolboxTrait;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Route;

/**
 * Defines an OpenApi Schema Generator for the Decoupled toolbox module.
 *
 * @OpenApiGenerator(
 *   id = "decoupled_toolbox",
 *   label = @Translation("Decoupled toolbox"),
 * )
 */
class DecoupledToolboxGenerator extends OpenApiGeneratorBase {

  use OpenapiDecoupledToolboxTrait;

  /**
   * {@inheritdoc}
   */
  public function getSpecification() {
    $definitions = $this->getDefinitions();
    $spec = [
      'openapi' => '3.0.0',
      'info' => $this->getInfo(),
      'servers' => [['url' => $this->request->getSchemeAndHttpHost()]],
      'security' => [$this->getSecurity()],
      'tags' => $this->getTags(),
      'components' => [
        'schemas' => $definitions,
        'securitySchemes' => $this->getSecurityDefinitions(),
      ],
      'consumes' => $this->getConsumes(),
      //  'produces' => $this->getProduces(),
      'paths' => $this->getPathsResponses($definitions),
    ];

    // Strip any empty arrays which aren't required.
    $required = ['openapi', 'info', 'paths'];
    foreach ($spec as $key => $item) {
      if (!in_array($key, $required) && is_array($item) && !count($item)) {
        unset($spec[$key]);
      }
    }
    return $spec;
  }

  /**
   * {@inheritdoc}
   */
  public function getDefinitions() {
    // @todo filter by bundle/ type.
    $bundle_name = isset($this->getOptions()['bundle_name']) ? $this->getOptions()['bundle_name'] : NULL;
    $entity_type_id = isset($this->getOptions()['entity_type_id']) ? $this->getOptions()['entity_type_id'] : NULL;
    static $definitions = [];
    if (!empty($definitions)) {
      return $definitions;
    }
    $entity_types = $this->getDecoupledToolboxEnabledEntityTypes($entity_type_id);
    foreach ($entity_types as $config_decoupled) {
      $this->buildDefintion($config_decoupled, $definitions);
    }
    return $definitions;
  }

  /**
   * Build the definition.
   *
   * @param \Drupal\openapi_decoupled_toolbox\Entity\OpenApiDecoupledToolbox $config_decoupled
   * @param array $definitions
   *
   * @return void
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function buildDefintion(OpenApiDecoupledToolbox $config_decoupled, array &$definitions) {
    foreach ($config_decoupled->getTargetBundle() as $bundle) {
      foreach ($config_decoupled->getTargetDisplay() as $display) {
        if (empty($display)) {
          continue;
        }
        $this->buildDefinitionEntityViewMode($config_decoupled->getTargetEntityType(), $bundle, $display, $definitions);
      }
    }
  }

  /**
   * Build definition entity view mode.
   *
   * @param $entity_type
   * @param $bundle
   * @param $display
   * @param array $definitions
   *
   * @return bool
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function buildDefinitionEntityViewMode($entity_type, $bundle, $display, array &$definitions) {
    /** @var  $view_mode \Drupal\Core\Entity\Entity\EntityViewDisplay */
    $view_mode = $this->entityTypeManager
      ->getStorage('entity_view_display')
      ->load($entity_type . '.' . $bundle . '.' . $display);
    if (empty($view_mode)) {
      return FALSE;
    }
    $definitions[$entity_type . '.' . $bundle . '.' . $display] = [
      'type' => 'object',
      'title' => t('@type @bundle @display schema', [
        '@type' => $entity_type,
        '@bundle' => $bundle,
        '@display' => $display,
      ]),
      'description' => t("Describes the payload for '@type' '@bundle' entities with display: @display.", [
        '@type' => $entity_type,
        '@bundle' => $bundle,
        '@display' => $display,
      ]),
    ];
    $this->buildProperties($entity_type, $definitions, $view_mode, $bundle, $display);
    return TRUE;
  }

  /**
   * Build the properties.
   *
   * @param $entity_type
   * @param array $definitions
   * @param \Drupal\field_layout\Entity\FieldLayoutEntityViewDisplay $view_mode
   * @param $bundle
   * @param $display
   *
   * @return false|void
   */
  private function buildProperties($entity_type, array &$definitions, EntityViewDisplay $view_mode, $bundle, $display) {
    if (empty($view_mode->get('content'))) {
      return FALSE;
    }
    $def_key = $entity_type . '.' . $bundle . '.' . $display;
    $field_defintiions = $view_mode->get('fieldDefinitions');
    // Loop on each field display in view mode.
    foreach ($view_mode->get('content') as $field_name => $content) {
      if (empty($content["settings"]["decoupled_field_key"])) {
        continue;
      }

      // Special case for entity references.
      $build_ref = $this->buildEntityReferenceProperties($content, $field_name, $field_defintiions, $definitions);

      // No re-location option.
      if (empty($content["settings"]["decoupled_field_location"])) {
        if ($build_ref) {
          $possibilities = $this->getReferencePossibility($content, $field_name, $field_defintiions);
          // Only one bundle type.
          if (count($possibilities) == 1) {
            $possibility = reset($possibilities);
            $definitions[$def_key]['properties'][$content["settings"]["decoupled_field_key"]] = [
              'type' => 'array',
              'items' => ['$ref' => '#/components/schemas/' . $possibility['type'] . '.' . $possibility['bundle'] . '.' . $possibility['display'],],
            ];
            continue;
          }
          $definitions[$def_key]['properties'][$content["settings"]["decoupled_field_key"]] = [
            'type' => 'array',
          ];
          foreach ($possibilities as $possibility) {
            $definitions[$def_key]['properties'][$content["settings"]["decoupled_field_key"]]['items']['oneOf']['$ref'] = '#/components/schemas/' . $possibility['type'] . '.' . $possibility['bundle'] . '.' . $possibility['display'];
          }
        }
        $definitions[$def_key]['properties'][$content["settings"]["decoupled_field_key"]] = [];
        $this->buildItemField($content, $field_name, $definitions[$def_key]['properties'][$content["settings"]["decoupled_field_key"]], $content["settings"]["decoupled_field_key"]);
        continue;
      }
      $location = explode('/', $content["settings"]["decoupled_field_location"]);

      $location = array_filter($location);
      if (empty($location)) {
        continue;
      }
      $loc_key = current($location);
      if (!empty($definitions[$def_key]['properties'][$loc_key])) {
        $temp_proper = $this->buildLocationProperties($content, $field_name, $location, $content["settings"]["decoupled_field_key"]);

        $definitions[$def_key]['properties'][$loc_key]['properties'] = array_merge($definitions[$def_key]['properties'][$loc_key]['properties'], $temp_proper['properties']);

        continue;
      }
      $definitions[$def_key]['properties'][current($location)] = $this->buildLocationProperties($content, $field_name, $location, $content["settings"]["decoupled_field_key"]);
    }
  }

  /**
   * Build properties for entity reference.
   *
   * @param $field
   * @param $field_name
   * @param array $field_defintiions
   * @param array $definitions
   *
   * @return bool
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function buildEntityReferenceProperties($field, $field_name, array $field_defintiions, array &$definitions) {
    $possibilities = $this->getReferencePossibility($field, $field_name, $field_defintiions);
    if ($possibilities === FALSE) {
      return FALSE;
    }
    foreach ($possibilities as $possibility) {
      $this->buildDefinitionEntityViewMode($possibility['type'], $possibility['bundle'], $possibility['display'], $definitions);
    }
    return TRUE;
  }

  /**
   * Get entity type and bund possibilities to be referenced by the field.
   *
   * @param array $field
   * @param $field_name
   * @param array $field_defintiions
   *
   * @return array|false
   */
  private function getReferencePossibility(array $field, $field_name, array $field_defintiions) {
    if ($field['type'] !== 'decoupled_entity_reference') {
      return FALSE;
    }
    // No view mode.
    if (empty($field["settings"]["decoupled_entity_view_mode"])) {
      return FALSE;
    }
    $display = $field["settings"]["decoupled_entity_view_mode"];
    // No field definitions.
    if (empty($field_defintiions[$field_name])) {
      return FALSE;
    }
    /** @var $field_defintiionssd \Drupal\field\Entity\FieldConfig */
    $settings_field = $field_defintiions[$field_name]->getSettings();
    // @todo look for default: how to tranform via Drupal.
    $target_type = $settings_field['handler'] ? str_replace('default:', '', $settings_field['handler']) : '';
    $bundles = $settings_field['handler_settings']['target_bundles'] ?? [];
    $result = [];
    foreach ($bundles as $bundle) {
      $result[] = [
        'type' => $target_type,
        'bundle' => $bundle,
        'display' => $display,
      ];
    }
    return $result;
  }

  /**
   * Build properties with re location option.
   *
   * @param array $field
   *   The content field of display.
   * @param string $field_name
   *   The drupal field name of the data.
   * @param array $location
   * @param $current_loc
   * @param $final_key
   *
   * @return array|string[]
   */
  private function buildLocationProperties($field, $field_name, array &$location, $final_key) {
    array_pop($location);
    $properties = [
      'type' => 'object',
    ];
    if (!empty($location)) {
      if (!empty($properties['properties'][current($location)])) {
        $temp_prop = $this->buildLocationProperties($field, $field_name, $location, $final_key);
        $properties['properties'][current($location)]['properties'] = array_merge($properties['properties'][current($location)]['properties'], $temp_prop['properties']);
        return $properties;
      }
      $properties['properties'][current($location)] = $this->buildLocationProperties($field, $field_name, $location, $final_key);
      return $properties;
    }
    if (empty($properties['properties'][$final_key])) {
      $properties['properties'][$final_key] = [];
    }
    // Get settings ouput formatter.
    $this->buildItemField($field, $field_name, $properties['properties'][$final_key], $final_key);

    return $properties;
  }

  /**
   * Build final item of properties.
   *
   * @param array $field
   *   The content field of display.
   * @param string $field_name
   *   The drupal field name of the data.
   * @param array $properties
   *   The properties array.
   * @param string $final_key
   *   The finale key alias.
   *
   */
  private function buildItemField(array $field, $field_name, array &$properties, $final_key) {
    $settings_ouput = $field['settings'][DecoupledFormatterBase::SETTINGS__DECOUPLED_FIELD_OUPUT];
    // Direct field type.
    if (!is_array($settings_ouput)) {

      $properties = [
        'type' => $settings_ouput ?? 'string',
        'title' => t('"@name" alias of field "@field"', [
          '@name' => $final_key,
          '@field' => $field_name,
        ]),
      ];
      return NULL;
    }
    // Array of only one type field.
    if (count($settings_ouput) == 1) {
      $properties = [
        'type' => 'array',
        'items' => [
          'type' => current($settings_ouput) ?? 'string',
          'title' => t('"@name" alias of field "@field"', [
            '@name' => $final_key,
            '@field' => $field_name,
          ]),
        ],
      ];
      return NULL;
    }
    // Array with keys.
    $properties = [
      'type' => 'object',
    ];
    foreach ($settings_ouput as $output_key => $ouput_type) {
      $properties[$output_key] = [
        'type' => $ouput_type,
        'title' => t('"@name" alias of field "@field"', [
          '@name' => $output_key,
          '@field' => $field_name,
        ]),
      ];
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getJsonSchema($described_format, $entity_type_id, $bundle_name = NULL) {
    if ($entity_type_id !== $bundle_name) {
      $schema = $this->schemaFactory->create($entity_type_id, $bundle_name);
    }
    else {
      $schema = $this->schemaFactory->create($entity_type_id);
    }

    if ($schema) {
      $json_schema = $this->serializer->normalize($schema, "schema_json:$described_format");
      unset($json_schema['$schema'], $json_schema['id']);
      $json_schema = $this->cleanSchema($json_schema);
      if (!$bundle_name) {
        // Add discriminator field.
        $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);
        if ($bundle_field = $entity_type->getKey('bundle')) {
          $json_schema['discriminator'] = $bundle_field;
        }
      }
    }
    else {
      $json_schema = [
        'type' => 'object',
        'title' => $this->t('@entity_type Schema', ['@entity_type' => $entity_type_id]),
        'description' => $this->t('Describes the payload for @entity_type entities.', ['@entity_type' => $entity_type_id]),
      ];
    }
    return $json_schema;
  }

  /**
   * {@inheritdoc}
   */
  public function getConsumes() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getProduces() {
    return $this->generateMimeTypesFromFormats($this->getDecoupledSupportedFormats());
  }

  /**
   * Get tags.
   */
  public function getTags() {
    $entity_types = $this->getDecoupledToolboxEnabledEntityTypes();
    /*
    $tags = [];
    foreach ($entity_types as $entity_type) {
      if ($this->includeEntityTypeBundle($entity_type->id())) {
        $tag = [
          'name' => $entity_type->id(),
          'description' => $this->t("Entity type: @label", ['@label' => $entity_type->getLabel()]),
          'x-entity-type' => $entity_type->id(),
          'x-definition' => [
            '$ref' => '#/definitions/' . $this->getEntityDefinitionKey($entity_type->id()),
          ],
        ];
        $tags[] = $tag;
      }
    }*/
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getPathsResponses(array $definitions) {
    $route = \Drupal::service('router.route_provider')
      ->getRouteByName('decoupled_toolbox.entity_decoupled_data.collection');

    $bundle_name = isset($this->getOptions()['bundle_name']) ? $this->getOptions()['bundle_name'] : NULL;

    foreach ($route->getMethods() as $method) {
      $open_api_method = strtolower($method);
      $path = $route->getPath();
      $path_method_spec = [];
      $formats = $this->getMethodSupportedFormats($route);
      $path_method_spec['parameters'] = [];
      if (!empty($formats)) {
        $format_parameter = [
          'name' => '_format',
          'in' => 'query',
          //  'type' => 'string',
          'enum' => $formats,
          'required' => TRUE,
          'description' => 'Request format',
        ];
        if (count($formats) == 1) {
          $format_parameter['default'] = $formats[0];
        }
        $path_method_spec['parameters'][] = $format_parameter;
      }

      $path_method_spec['responses'] = $this->getErrorResponses();

      $path_method_spec['responses'][Response::HTTP_OK] = [
        'description' => 'A list of entities',
        'content' => [
          "application/json" => [
            'schema' => [
              'oneOf' => $this->getRefSchemaEntityType($definitions),
            ],
          ],
        ],
      ];
      $path_method_spec['summary'] = t('Decoupled toolbox endpoints');
      $path_method_spec['parameters'] = array_merge($path_method_spec['parameters'], $this->getRouteParameters($route), $this->getQueryParameter());

      $path_method_spec['operationId'] = "decoupled_toolbox:" . $method;
      // $path_method_spec['schemes'] = [$this->request->getScheme()];

      $path_method_spec['security'] = $this->getRouteSecurity($route);
      $api_paths[$path][$open_api_method] = $path_method_spec;

    }
    return $api_paths;
  }

  /**
   * Build array of ref schema entities for 200 status response.
   *
   * @param array $definitions
   *   Array of entities definitions.
   *
   * @return array
   */
  protected function getRefSchemaEntityType(array $definitions) {
    $ref = [];

    $decoupled_toolbox_entity = $this->getDecoupledToolboxEnabledEntityTypes();
    foreach ($definitions as $key_def => $definition) {
      $find_def = FALSE;
      // Display only the entity set on config.
      foreach ($decoupled_toolbox_entity as $decoupled) {
        $entity_type = $decoupled->getTargetEntityType();
        $bundles = $decoupled->getTargetBundle();
        $displays = $decoupled->getTargetDisplay();
        foreach ($bundles as $bundle) {
          foreach ($displays as $display) {
            if ($key_def == $entity_type . '.' . $bundle . '.' . $display) {
              $find_def = TRUE;
              break(3);
            }
          }
        }
      }
      if ($find_def) {
        $ref[] = ['$ref' => '#/components/schemas/' . $key_def];
      }
    }
    return $ref;
  }

  /**
   * Get the error responses.
   *
   * @see https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#responseObject
   *
   * @return array
   *   Keys are http codes. Values responses.
   */
  protected function getErrorResponses() {
    $responses[Response::HTTP_BAD_REQUEST] = [
      'description' => Response::$statusTexts[Response::HTTP_BAD_REQUEST],
      'content' => [
        'text/plain' => [
          'schema' => [
            'type' => 'string',
            'example' => 'offset parameter is undefined.',
          ],
        ],
      ],
    ];

    $responses[Response::HTTP_NOT_FOUND] = [
      'description' => Response::$statusTexts[Response::HTTP_NOT_FOUND],
      'content' => [
        'text/plain' => [
          'schema' => [
            'type' => 'string',
            'example' => 'Could not retrieve content',
          ],
        ],
      ],
    ];
    $responses[Response::HTTP_NOT_IMPLEMENTED] = [
      'description' => Response::$statusTexts[Response::HTTP_NOT_IMPLEMENTED],
      'content' => [
        'text/plain' => [
          'schema' => [
            'type' => 'string',
            'example' => 'Unavailable decoupled view display',
          ],
        ],
      ],
    ];
    $responses[Response::HTTP_INTERNAL_SERVER_ERROR] = [
      'description' => Response::$statusTexts[Response::HTTP_INTERNAL_SERVER_ERROR],
      'content' => [
        'text/plain' => [
          'schema' => [
            'type' => 'string',
            'example' => Response::$statusTexts[Response::HTTP_INTERNAL_SERVER_ERROR],
          ],
        ],
      ],
    ];
    return $responses;
  }

  /**
   * Get OpenAPI parameters for a route.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route.
   *
   * @return array
   *   The resource parameters.
   */
  protected function getRouteParameters(Route $route) {
    $config_decoupleds = $this->getDecoupledToolboxEnabledEntityTypes();
    $enum = ['bundle' => [], 'type' => []];
    foreach ($config_decoupleds as $config_decoupled) {
      $enum['bundle'] = array_merge($enum['bundle'], $config_decoupled->getTargetBundle());
      $enum['type'] = array_merge($enum['type'], [$config_decoupled->getTargetEntityType()]);
    }
    $parameters = [];
    $vars = $route->compile()->getPathVariables();
    foreach ($vars as $var) {
      $schema = ['type' => 'string'];
      if (!empty($enum[$var])) {
        $schema['enum'] = array_values($enum[$var]);
      }
      $parameters[] = [
        'name' => $var,
        'schema' => $schema,
        'in' => 'path',
        'required' => TRUE,
      ];

    }
    return $parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function getApiName() {
    return $this->t('Decoupled toolbox API');
  }

  /**
   * {@inheritdoc}
   */
  protected function getApiDescription() {
    return $this->t('The Decoupled Toolobx API provide by the Decoupled toolbox module.');
  }

  /**
   * @param \Symfony\Component\Routing\Route $route
   *
   * @return array|string[]
   */
  protected function getMethodSupportedFormats(Route $route) {
    if ($formats = $route->getRequirement('_format')) {
      return explode('|', $formats);
    }

    return [];
  }

  /**
   * @param \Symfony\Component\Routing\Route $route
   *
   * @return array
   */
  protected function getRouteSecurity(Route $route) {
    //@todo _auth
    if ($auths = $route->getOption('_auth')) {
      foreach ($auths as $auth) {
        $security[] = [$auth => []];
      }
      return $security;
    }
    return [];
  }

  /**
   * Generate list of MIME Types based on a list of serializer formats.
   *
   * @param array $formats
   *   List of formats.
   *
   * @return array
   *   List of MIME Types based on $formats.
   *   The list is MIME Types are on the same order as the inserted $format
   */
  protected function generateMimeTypesFromFormats(array $formats) {
    $mime_types = [];
    foreach ($formats as $format) {
      $mime_types[] = 'application/' . preg_replace('/_/', '+', trim(strtolower($format)));
    }
    return $mime_types;
  }

  /**
   * Returns a list of supported Format on Decoupled toobox.
   *
   * @return array
   *   The list of supported formats.
   */
  protected function getDecoupledSupportedFormats() {
    return ['json'];
  }

}

==> decoupled_toolbox-1.4.x-dev.tar.gz: decoupled_toolbox/modules/openapi_decoupled_toolbox/src/Plugin/openapi/OpenApiGenerator/DecoupledToolboxGenerator.php <==
<?php

namespace Drupal\openapi_decoupled_toolbox\Plugin\openapi\OpenApiGenerator;

use Symfony\Component\Routing\Route;

/**
 *
 */
class DecoupledToolboxGenerator extends DecoupledToolboxGeneratorBase {

  /**
   * {@inheritdoc}
   */
  public function getPathsResponses(array $definitions) {
    $route = \Drupal::service('router.route_provider')
      ->getRouteByName('decoupled_toolbox.entity_decoupled_data.collection');

    $bundle_name = isset($this->getOptions()['bundle_name']) ? $this->getOptions()['bundle_name'] : NULL;
    $entity_type_id = isset($this->getOptions()['entity_type_id']) ? $this->getOptions()['entity_type_id'] : NULL;
    $config_decoupleds = $this->getDecoupledToolboxEnabledEntityTypes();

    foreach ($route->getMethods() as $method) {
      foreach ($config_decoupleds as $config_decoupled) {
        if (!empty($entity_type_id) && $config_decoupled->getTargetEntityType() != $entity_type_id) {
          continue;
        }
        $path = $route->getPath();
        $path = str_replace('{type}', $config_decoupled->getTargetEntityType(), $path);
        foreach ($config_decoupled->getTargetBundle() as $bundle) {
          if (!empty($bundle_name) && $bundle != $bundle_name) {
            continue;
          }
          $open_api_method = strtolower($method);
          $path = str_replace('{bundle}', $bundle, $path);
          $formats = $this->getMethodSupportedFormats($route);
          $path_method_spec = $this->generatePathMethod($config_decoupled, $bundle, $method, $definitions, $formats);
          // $path_method_spec['schemes'] = [$this->request->getScheme()];

          $path_method_spec['security'] = $this->getRouteSecurity($route);
          $api_paths[$path][$open_api_method] = $path_method_spec;

        }
      }
    }
    return $api_paths;
  }

  /**
   * Get OpenAPI parameters for a route.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route.
   *
   * @return array
   *   The resource parameters.
   */
  protected function getRouteParameters(Route $route) {
    $config_decoupleds = $this->getDecoupledToolboxEnabledEntityTypes();
    $enum = ['bundle' => [], 'type' => []];
    foreach ($config_decoupleds as $config_decoupled) {
      $enum['bundle'] = array_merge($enum['bundle'], $config_decoupled->getTargetBundle());
      $enum['type'] = array_merge($enum['type'], [$config_decoupled->getTargetEntityType()]);
    }
    $parameters = [];
    $vars = $route->compile()->getPathVariables();
    foreach ($vars as $var) {
      $schema = ['type' => 'string'];
      if (!empty($enum[$var])) {
        $schema['enum'] = array_values($enum[$var]);
      }
      $schema['enum'] = array_unique($schema['enum']);
      $parameters[] = [
        'name' => $var,
        'schema' => $schema,
        'in' => 'path',
        'required' => TRUE,
      ];

    }
    return $parameters;
  }

  /**
   * @param \Symfony\Component\Routing\Route $route
   *
   * @return array|string[]
   */
  protected function getMethodSupportedFormats(Route $route) {
    if ($formats = $route->getRequirement('_format')) {
      return explode('|', $formats);
    }

    return [];
  }

  /**
   * @param \Symfony\Component\Routing\Route $route
   *
   * @return array
   */
  protected function getRouteSecurity(Route $route) {
    //@todo _auth
    if ($auths = $route->getOption('_auth')) {
      foreach ($auths as $auth) {
        $security[] = [$auth => []];
      }
      return $security;
    }
    return [];
  }

  /**
   * Generate list of MIME Types based on a list of serializer formats.
   *
   * @param array $formats
   *   List of formats.
   *
   * @return array
   *   List of MIME Types based on $formats.
   *   The list is MIME Types are on the same order as the inserted $format
   */
  protected function generateMimeTypesFromFormats(array $formats) {
    $mime_types = [];
    foreach ($formats as $format) {
      $mime_types[] = 'application/' . preg_replace('/_/', '+', trim(strtolower($format)));
    }
    return $mime_types;
  }

  /**
   * Returns a list of supported Format on Decoupled toobox.
   *
   * @return array
   *   The list of supported formats.
   */
  protected function getDecoupledSupportedFormats() {
    return ['json'];
  }

}
